#!/usr/bin/perl

package PDF;

use pdflib 2.0;
use strict;
use Carp;

my (%pagesize, @fontnames, %colornames, %defaults);
use vars qw($AUTOLOAD);

sub BEGIN {
  #
  # predefined page sizes
  #
  %pagesize = ( "A0" => [2380, 3368],
		"A1" => [1684, 2380],
		"A2" => [1190, 1684],
		"A3" => [842,  1190],
		"A4" => [595,   842],
		"A5" => [421,   595],
		"A6" => [297,   421],
		"B5" => [501,   709],
		"LETTER"=> [612,792],
		"LEGAL" => [612, 1008],
		"LEDGER" => [1224, 792] );
  #
  # The standard fonts
  #
  @fontnames = ("Courier", "Courier-Bold", "Courier-Oblique", "Courier-BoldOblique",
		"Helvetica", "Helvetica-Bold", "Helvetica-Oblique", 
		"Helvetica-BoldOblique",
		"Times-Roman","Times-Bold", "Times-Italic", "Times-BoldItalic",
		"Symbol", "ZapfDingbats");
  #
  # some named colors
  #
  %colornames = ( "white"   => [1., 1., 1.],
		  "black"   => [0., 0., 0.],
		  "red"     => [1., 0., 0.],
		  "green"   => [0., 1., 0.],
		  "blue"    => [0., 0., 1.],
		  "cyan"    => [0., 1., 1.],
		  "magenta" => [1., 0., 1.],
		  "yellow"  => [1., 1., 0. ],
		  );
  %defaults = (
	       '_gray_fill'         => 0.,
	       '_gray_stroke'       => 0.,
	       '_rgb_fill'          => $colornames{'black'},
	       '_rgb_stroke'        => $colornames{'black'},
	       '_text_horiz_scaling'=> 100.0,
	       '_text_rendering'    => 0,
	       '_text_matrix'       => [ 1.,0.,0.,1.,0.,0.],
	       '_char_spacing'      => 0.,
	       '_word_spacing'      => 0.,
	       '_flat'              => 0.,
	       '_linejoin'          => 0,
	       '_linecap'           => 0,
	       '_miterlimit'        => 10.0,
	       '_linewidth'         => 1.0,
	       '_fillrule'          => 'winding',
	      );

} # end of BEGIN part

sub AUTOLOAD {
  no strict "refs";
  if ($AUTOLOAD =~ /.*::get(_\w+)/ ) {
    my $attr_name = $1;
    *{$AUTOLOAD} = sub {return $_[0]->{$attr_name}};
    return $_[0]->{$attr_name};
  }
  croak "No such method: $AUTOLOAD";
}


sub new {
  my $class = $_[0];
  my $p = pdflib::PDF_new();
#  pdflib::PDF_set_parameter($p,"debug","abcdefghijklmnopqrstuvwxyz");
  bless { '_pdf' => $p } , $_[0];
}

sub open_file {
  my $self = $_[0];
  my $f = $_[1];
  $self->{'_filename'} = $f;
  my $ret = pdflib::PDF_open_file($self->{'_pdf'},$f);
  return $ret;
}

sub set_info {
  my ($self, $key, $value) = @_;
  pdflib::PDF_set_info($self->{'_pdf'}, $key, $value);
  $self->{'_info'}->{$key} = $value;
}

sub set_parameter {
  my ($self,$key,$value) = @_;
  pdflib::PDF_set_parameter($self->{'_pdf'},$key,$value);
}


sub begin_page {
  my ($self,$a,$b) = @_;
  if ($a && $b) {
    $self->{'_page'} = [ $a, $b];
    pdflib::PDF_begin_page($self->{'_pdf'}, $a, $b);
  } elsif (exists ($pagesize{uc($a)}) ) {
    $a = uc($a);
    $self->{'_page'} = $pagesize{$a};
    pdflib::PDF_begin_page($self->{'_pdf'}, 
			   $pagesize{$a}->[0], $pagesize{$a}->[1]);
  } else {
    $self->{'_page'} = $pagesize{'A4'};
    pdflib::PDF_begin_page($self->{'_pdf'}, 
			   $pagesize{'A4'}->[0], $pagesize{'A4'}->[1]);
    warn "Illegal pagesize $a, using A4";
  }
  foreach ( keys %defaults) {
    $self->{$_} = $defaults{$_};
  }
}

sub end_page {
  pdflib::PDF_end_page($_[0]->{'_pdf'});
  delete($_[0]->{'_page'});
}

#sub get_page {
#  return $_[0]->{'_page'};
#}

sub closefile {
  pdflib::PDF_close($_[0]->{'_pdf'});
}

sub deleteobj {
  pdflib::PDF_delete($_[0]->{'_pdf'});
}
#
# Line attributes
# 
sub set_dash {
  my ($self, $b, $w) = @_;
  pdflib::PDF_setdash($self->{'_pdf'},$b, $w);
  $self->{'_dash'} = [ $b, $w];
}

sub set_linewidth {
  my ($self, $w) = @_;
  pdflib::PDF_setlinewidth($self->{'_pdf'},$w);
  $self->{'_linewidth'}=$w;
}

sub set_flat {
  my ($self, $f) = @_;
  pdflib::PDF_setflat($self->{'_pdf'},$f);
  $self->{'_flat'}=$f;
}

sub set_linejoin {
  my ($self, $j) = @_;
  pdflib::PDF_setlinejoin($self->{'_pdf'},$j);
  $self->{'_linejoin'}=$j;
}

sub set_linecap {
  my ($self, $c) = @_;
  pdflib::PDF_setflat($self->{'_pdf'},$c);
  $self->{'_linecap'}=$c;
}

sub set_miterlimit {
  my ($self, $m) = @_;
  pdflib::PDF_setflat($self->{'_pdf'},$m);
  $self->{'_miterlimit'}=$m;
}

#
# COLORs
#
sub set_gray_fill{
  my ($self, $g) = @_;
  pdflib::PDF_setgray_fill($self->{'_pdf'}, $g);
  $self->{'_gray_fill'} = $g;
}

sub set_gray_stroke{
  my ($self, $g) = @_;
  pdflib::PDF_setgray_stroke($self->{'_pdf'}, $g);
  $self->{'_gray_stroke'} = $g;
}
sub set_gray{
  my ($self, $g) = @_;
  pdflib::PDF_setgray($self->{'_pdf'}, $g);
  $self->{'_gray_fill'} = $g;
  $self->{'_gray_stroke'} = $g;
}

sub get_color {
  my ($c1, $c2, $c3) = @_;
  my $retval;
  if (! defined($c2) && exists($colornames{$c1})) {
    $retval = $colornames{$c1};
  } elsif (! defined($c2) ) {
    $retval = ref($c1) eq "ARRAY" ? $c1 : $colornames{'black'};
  } else {
    $retval = [$c1, $c2, $c3];
  }
  return $retval;
}

sub set_rgbcolor_fill {
  my ($self, $c1, $c2, $c3) = @_;
  my $colref = get_color($c1, $c2, $c3);
  pdflib::PDF_setrgbcolor_fill($self->{'_pdf'}, 
			       $colref->[0],$colref->[1],$colref->[2]);
  $self->{'_rgb_fill'} = $colref;
}

sub set_rgbcolor_stroke {
  my ($self, $c1, $c2, $c3) = @_;
  my $colref = get_color($c1, $c2, $c3);
  pdflib::PDF_setrgbcolor_stroke($self->{'_pdf'}, 
				 $colref->[0],$colref->[1],$colref->[2]);
  $self->{'_rgb_stroke'} = $colref;
}

sub set_rgbcolor {
  my ($self, $c1, $c2, $c3) = @_;
  my $colref = get_color($c1, $c2, $c3);
  pdflib::PDF_setrgbcolor($self->{'_pdf'}, 
			  $colref->[0],$colref->[1],$colref->[2]);
  $self->{'_rgb_fill'} = $colref;
  $self->{'_rgb_stroke'} = $colref;
}

#
# State stuff
#
sub save {
  my ($self) = @_;
  pdflib::PDF_save($self->{'_pdf'});
}

sub restore {
  my ($self) = @_;
  pdflib::PDF_restore($self->{'_pdf'});
}

sub translate {
  my ($self,$x, $y) = @_;
  pdflib::PDF_translate($self->{'_pdf'}, $x, $y);
  $self->{'_translate'} = [$x, $y];
}

sub scale {
  my ($self,$x, $y) = @_;
  pdflib::PDF_scale($self->{'_pdf'}, $x, $y);
  $self->{'_scale'} = [$x, $y];
}

sub rotate {
  my ($self,$x) = @_;
  pdflib::PDF_rotate($self->{'_pdf'}, $x);
  $self->{'_rotate'} = $x;
}

#
# Text, fonts etc
#
sub set_font {
  my $self = $_[0];
  my $font = $_[1];
  my $size = $_[2];
  my $encoding = $_[3] || "default";
  my $pdf = $self->{'_pdf'};
  my $embed = grep { /$font/o } @fontnames ? 1 : 0;
  my $fullname = $font . "#" . $encoding . "#" . $embed;
  my $fontnr;
  if (exists $self->{'_fonts'}->{$fullname}) {
    $fontnr = $self->{'_fonts'}->{$fullname};
  } else {
    $fontnr = pdflib::PDF_findfont($pdf,$font,$encoding, $embed);
    $self->{'_fonts'}->{$fullname} = $fontnr;
  }
  pdflib::PDF_setfont($pdf,$fontnr,$size);
  $self->{'_fontsize'} = $size;
  $self->{'_font'} = $fontnr;
  $self->{'_curfont'} = [$font , $fontnr ,$size];
}

sub text {
  my ($self,$text, $x, $y) = @_;
  if ($x && $y) {
    set_text_pos($self, $x, $y);
  }
  pdflib::PDF_show($self->{'_pdf'},$text);
  
  my $width = pdflib::PDF_stringwidth($self->{'_pdf'},$text,
				      $self->{'_curfont'}->[1],
				      $self->{'_curfont'}->[2]);
  $self->{'_lasttextwidth'}= $width;
}

sub continue_text {
  my ($self, $text) = @_;
  pdflib::PDF_continue_text($self->{'_pdf'},$text);
}

sub set_leading {
  my ($self, $lead) = @_;
  pdflib::PDF_set_leading($self->{'_pdf'},$lead);
  $self->{'_leading'} = $lead;
}

sub set_text_rise {
  my ($self, $rise) = @_;
  pdflib::PDF_set_text_rise($self->{'_pdf'},$rise);
  $self->{'_rise'} = $rise;
}

sub set_text_horiz_scaling {
  my ($self, $scale) = @_;
  pdflib::PDF_set_text_horiz_scaling($self->{'_pdf'},$scale);
  $self->{'_text_horiz_scaling'} = $scale;
}

sub set_text_rendering {
  my ($self, $mode) = @_;
  pdflib::PDF_set_text_rendering($self->{'_pdf'},$mode);
  $self->{'_text_rendering'} = $mode;
}

sub set_text_matrix {
  my ($self, @matrix) = @_;
  pdflib::PDF_set_text_matrix($self->{'_pdf'},@matrix);
  $self->{'_text_matrix'} = @matrix;
}

sub set_char_spacing {
  my ($self, $space) = @_;
  pdflib::PDF_set_char_spacing($self->{'_pdf'},$space);
  $self->{'_char_spacing'} = $space;
}

sub set_word_spacing {
  my ($self, $space) = @_;
  pdflib::PDF_set_word_spacing($self->{'_pdf'},$space);
  $self->{'_word_spacing'} = $space;
}

sub get_textwidth {
  my ($self, $text) = @_;
  my $font = pdflib::PDF_get_font($self->{'_pdf'});
  my $size = pdflib::PDF_get_fontsize($self->{'_pdf'});
  my $retval = pdflib::PDF_stringwidth($self->{'_pdf'},$text,
				 $font, $size);
  return $retval;
}


sub set_text_pos{
  my ($self, $x, $y) = @_;
  pdflib::PDF_set_text_pos($self->{'_pdf'}, $x, $y);
  $self->{'_textpos'} = [$x, $y];
  $self->{'_nexttextpos'} = [$x, $y];
}

#
# PATH stuff
#

sub moveto {
  my ($self, $x, $y) = @_;
  pdflib::PDF_moveto($self->{'_pdf'},$x, $y);
}

sub lineto {
  my ($self, $x, $y) = @_;
  pdflib::PDF_lineto($self->{'_pdf'},$x, $y);

}

sub closepath {
  my ($self) = @_;
  pdflib::PDF_closepath($self->{'_pdf'});
}

sub closepath_stroke {
  my ($self) = @_;
  pdflib::PDF_closepath_stroke($self->{'_pdf'});
}

sub closepath_fill {
  my ($self) = @_;
  pdflib::PDF_closepath_fill($self->{'_pdf'});
}

sub closepath_fill_stroke {
  my ($self) = @_;
  pdflib::PDF_closepath_fill_stroke($self->{'_pdf'});
}


sub curveto {
  my ($self, $x1, $y1, $x2, $y2, $x3, $y3) = @_;
  pdflib::PDF_curveto($self->{'_pdf'},$x1, $y1, $x2, $y2, $x3, $y3);
}

sub circle {
  my ($self, $x, $y, $r) = @_;
  pdflib::PDF_circle($self->{'_pdf'},$x, $y,$r);
}

sub arc {
  my ($self, $x, $y, $r, $alpha1, $alpha2) = @_;
  pdflib::PDF_arc($self->{'_pdf'},$x, $y, $r, $alpha1, $alpha2);
}


sub fill_circle {
  my ($self, $x, $y, $r) = @_;
  pdflib::PDF_circle($self->{'_pdf'},$x, $y,$r);
  pdflib::PDF_fill($self->{'_pdf'});
}

sub line {
  my ($self, $x1, $y1,$x2, $y2) = @_;
  my $pdf = $self->{'_pdf'};
  pdflib::PDF_moveto($pdf,$x1, $y1);
  pdflib::PDF_lineto($pdf,$x2, $y2);
  pdflib::PDF_stroke($pdf);
}

sub rect {
  my ($self, $x, $y, $w, $h) = @_;
  pdflib::PDF_rect($self->{'_pdf'},$x, $y,$w, $h);
}

sub fill_rect {
  pdflib::PDF_rect($_[0]->{'_pdf'},$_[1],$_[2],$_[3],$_[4]);
  pdflib::PDF_fill($_[0]->{'_pdf'});
}

sub fill {
  pdflib::PDF_fill($_[0]->{'_pdf'});
}

sub stroke {
  pdflib::PDF_stroke($_[0]->{'_pdf'});
}

sub clip {
  pdflib::PDF_clip($_[0]->{'_pdf'});
}

sub endpath {
  pdflib::PDF_endpath($_[0]->{'_pdf'});
}

sub set_fillrule {
  my ($self, $rule) = @_;
  pdflib::PDF_set_fillrule($self->{'_pdf'}, $rule);
  $self->{'_fillrule'} = $rule;
}

#
# IMAGE stuff
#
sub set_image_size{
  my ($self, $image) = @_;
  push @{$self->{'_images'}{$image}}, 
  pdflib::PDF_get_image_width($self->{'_pdf'},$image);
  push @{$self->{'_images'}{$image}}, 
  pdflib::PDF_get_image_height($self->{'_pdf'},$image);
}

sub get_image_width {
  return $_[0]->{'_images'}{$_[1]}[0];
}

sub get_image_width {
  return $_[0]->{'_images'}{$_[1]}[1];
}

sub open_JPEG {
  my ($self, $file) = @_;
  my $retval= pdflib::PDF_open_JPEG($self->{'_pdf'},$file);
  if ($retval >= 0) {
    $self->{'_images'}{$retval}=["JPEG",$file];
    set_image_size($self,$retval);
  }
  return $retval;
}

sub open_GIF {
  my ($self, $file) = @_;
  my $retval= pdflib::PDF_open_GIF($self->{'_pdf'},$file);
  if ($retval >= 0) {
    $self->{'_images'}{$retval}=["GIF",$file];
    set_image_size($self,$retval);
  }
  return $retval;
}
sub open_TIFF {
  my ($self, $file) = @_;
  my $retval= pdflib::PDF_open_TIFF($self->{'_pdf'},$file);
  if ($retval >= 0) {
    $self->{'_images'}{$retval}=["TIFF",$file];
    set_image_size($self,$retval);
  }
  return $retval;
}

sub open_CCITT {
  my ($self, $file, $w, $h, $rev, $k, $black) = @_;
  my $retval = pdflib::PDF_open_CCITT($self->{'_pdf'},$file,
				       $w, $h, $rev,$k, $black);
  if ($retval >= 0) {
    $self->{'_images'}{$retval}=["CCITT",$file];
    set_image_size($self, $retval);
  }
  return $retval;
}

sub place_image{
  my ($self, $image, $x, $y, $f) = @_;
  pdflib::PDF_place_image($self->{'_pdf'},$image, $x, $y,$f || 1.0);
}

sub place_inline_image{
  my ($self, $image, $x, $y, $f) = @_;
  pdflib::PDF_place_inline_image($self->{'_pdf'},$image, $x, $y,$f || 1.0);
}

sub put_image {
  my ($self, $image) = @_;
  pdflib::PDF_put_image($self->{'_pdf'}, $image);
}
sub get_image_size {
  my ($self, $image) = @_;
  my @size;
  push @size, @{$self->{'_images'}{$image}}[2];
  push @size, @{$self->{'_images'}{$image}}[3];
  return \@size;
}

sub close_image {
  my ($self, $image) = @_;
  pdflib::PDF_close_image($self->{'_pdf'},$image);
}

sub DESTROY {
#  print  "DESTROY\n";
#  my $self = @_;
#  my $pdf = $self->{'_pdf'};
  if (exists $_[0]->{'_page'}) {
#    print "calling end_page\n";
    pdflib::PDF_end_page($_[0]->{'_pdf'});
    delete($_[0]->{'_page'});
  }
#  print "calling close\n";
  pdflib::PDF_close($_[0]->{'_pdf'});
#  print "calling destroy\n";
  pdflib::PDF_destroy($_[0]->{'_pdf'});
}

1;
